{----------------------------------------------------}
{ Andreis LCARS-Components 1.1 for Delphi (2, 3), 4 }
{----------------------------------------------------}
{ Copyright (c) 1999 Andrei Scheibner                }
{ http://www.htwm.de/~ascheibn                       }
{ ascheibn@htwm.de                                   }
{----------------------------------------------------}
{These Components are Freeware. You can distribute them}
{altogether, you can improve them , but I would appreciate}
{it, if my name appears somewhere in your improvements.}

unit LCARSPanel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;


type
  TCaptionPosition = (cpCenter, cpTopLeft, cpTopRight, cpBottomRight, cpBottomLeft);
  TLCARSPanel = class(TGraphicControl)
  private
   FBack : TBitmap;
   FBackColor : TColor;
   FCaption : TCaption;
   FCaptionWidth : integer;
   FCaptionHeight : integer;
   FCaptionPosX : integer;
   FCaptionPosY : integer;
   FCaptionMarginX : integer;
   FCaptionMarginY : integer;
   FCaptionPosition : TCaptionPosition;
   FMouseInControl: Boolean;
   FOnMouseEnter: TNotifyEvent;
   FOnMouseLeave: TNotifyEvent;
   procedure SetCaption(value : TCaption);
   procedure SetCaptionPosition(value : TCaptionPosition);
   procedure SetCaptionMarginX(value : integer);
   procedure SetCaptionMarginY(value : integer);
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    procedure Paint; override;
    procedure MouseEnter; dynamic;
    procedure MouseLeave; dynamic;
  public
    property MouseInControl: Boolean read FMouseInControl;
    constructor Create(AOwner : TComponent) ; override;
    destructor Destroy; override;
  published
   property Anchors;
   property Constraints;
   property Parent;
   property Width default 129;
   property Height default 41;
   property Visible default true;
   property Enabled default true;
   property Font;
   property ParentFont default true;
   property Hint;
   property ShowHint;
   property ParentShowHint;
   property Align;
   property CaptionPosition : TCaptionPosition read FCaptionPosition
    write SetCaptionPosition default cpCenter;
   property Caption : TCaption read FCaption
    write SetCaption;
   property CaptionMarginX : integer read FCaptionMarginX
    write SetCaptionMarginX default 2;
   property CaptionMarginY : integer read FCaptionMarginY
    write SetCaptionMarginY default 0;
   property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
   property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
   property Color default clyellow;
   property OnClick;
   property OnDblClick;
   property OnMouseDown;
   property OnMouseUp;
   property OnMouseMove;
   property OnDragDrop;
   property OnDragOver;
   property OnEndDrag;
  end;

procedure Register;

implementation

constructor TLCARSPanel.Create(AOwner : TComponent);
begin
 inherited Create(AOwner);
 Width := 80;
 Height := 40;
 Color := clyellow;
 FBackColor := clBlack;
 Visible := true;
 CaptionPosition := cpCenter;
 ControlStyle:=ControlStyle+[csOpaque];
 FBack := TBitmap.Create;
 CaptionMarginX := 2;
 CaptionMarginY := 0;
 ParentFont := true;
end;

destructor TLCARSPanel.Destroy;
begin
 FBack.Free;
 inherited Destroy;
end;

procedure TLCARSPanel.Paint;
var VisibleRect, BackRect : TRect;
begin
 FBack.Canvas.Font.Assign(Font);
 FBack.Width := Width;
 FBack.Height := Height;
 if Parent <> nil then
  with Parent do
   FBackColor := Color;
 FCaptionWidth := FBack.Canvas.TextWidth(FCaption);
 FCaptionHeight := FBack.Canvas.TextHeight(FCaption);
 FBack.Canvas.Brush.Color := FBackColor;
 FBack.Canvas.Pen.Color := FBackColor;
 FBack.Canvas.Rectangle(0, 0, FBack.Width, FBack.Height);
 FBack.Canvas.Brush.Color := Color;
 FBack.Canvas.Pen.Color := Color;
 FBack.Canvas.Rectangle(0,0, FBack.width, FBack.height);
 case FCaptionPosition of
  cpCenter :
   begin
    FCaptionPosX := (FBack.Width div 2) - (FCaptionWidth div 2);
    FCaptionPosY := (FBack.Height div 2) - (FCaptionHeight div 2);
   end;
  cpTopLeft :
   begin
    FCaptionPosX := CaptionMarginX;
    FCaptionPosY := CaptionMarginY;
   end;
  cpTopRight :
   begin
    FCaptionPosX := FBack.Width - CaptionMarginX - FCaptionWidth;
    FCaptionPosY := CaptionMarginY;
   end;
  cpBottomRight :
   begin
    FCaptionPosX := FBack.Width - CaptionMarginX - FCaptionWidth;
    FCaptionPosY := FBack.Height - CaptionMarginY - FCaptionHeight;
   end;
  cpBottomLeft :
   begin
    FCaptionPosX := CaptionMarginX;
    FCaptionPosY := FBack.Height - CaptionMarginY - FCaptionHeight;
   end;
 end;
 FBack.Canvas.TextOut(FCaptionPosX, FCaptionPosY, FCaption);
 VisibleRect := Rect(0, 0, Width, Height);
 BackRect := Rect(0, 0, FBack.Width, Fback.Height);
 Canvas.CopyRect(VisibleRect, FBack.Canvas, BackRect);
end;

procedure TLCARSPanel.SetCaption(value : TCaption);
begin
 if FCaption <> value then
  begin
   FCaption := value;
   Repaint;
  end;
end;

procedure TLCARSPanel.SetCaptionPosition(value : TCaptionPosition);
begin
 if FCaptionPosition <> value then
  begin
   FCaptionPosition := value;
   Repaint;
  end;
end;

procedure TLCARSPanel.SetCaptionMarginX(value : integer);
begin
 if FCaptionMarginX <> value then
  begin
   FCaptionMarginX := value;
   Repaint;
  end;
end;

procedure TLCARSPanel.SetCaptionMarginY(value : integer);
begin
 if FCaptionMarginY <> value then
  begin
   FCaptionMarginY := value;
   Repaint;
  end;
end;

procedure TLCARSPanel.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if not FMouseInControl and Enabled {and IsForegroundTask } then begin
    FMouseInControl := True;
    MouseEnter;
  end;
end;

procedure TLCARSPanel.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if FMouseInControl and Enabled then begin
    FMouseInControl := False;
    MouseLeave;
  end;
end;

procedure TLCARSPanel.MouseEnter;
begin
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

procedure TLCARSPanel.MouseLeave;
begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

procedure Register;
begin
  RegisterComponents('LCARS-Comps', [TLCARSPanel]);
end;

end.
